const fs = require('fs');
const path = require('path');
const DateHandler = require('../functions/DateHandler');
const ms = require('ms');
const moment = require('moment');

class Logger {
  cleanUpTaskIntervalId = null;
  logFolder = process.env.LOG_FOLDER;
  keepLogDays = process.env.KEEP_LOG_DAYS;
  constructor(config = {}) {
    const {
      fileName = 'log',
      filePath = null,
      fileExtension = 'txt',
      useTimeStamp = true
    } = config;
    this.fileExtension = fileExtension;
    this.filePath = filePath ? filePath : this.getDefaultFilePath();
    this.fileName = fileName;
    this.useTimeStamp = useTimeStamp;
  }

  getErrorMessage(error) {
    if(typeof error === 'string') return error;
    if(error instanceof Error) return `${error.message} ${error.stack}`;
    return `Unexpected Error Structure ${error}`;
  }

  getHandledText({label, value}) {
    if(!value) return '';
    const text = typeof value === 'object' ? JSON.stringify(value) : value;
    return `${label}: ${text}\n`;
  }

  getDefaultFilePath() {
    if(process.env.NODE_ENV === 'development') {
      return this.logFolder;
    } else {
      return path.join(this.logFolder, process.argv[2]);
    }
  }

  format(error, info) {
    const sysdate = DateHandler.toString({ useTime: true });
    return `[${sysdate}] ${error}\n${info}\n`;
  }

  async removeFile(filePath, fileName) {
    return new Promise((resolve, reject) => {
      fs.unlink(path.join(filePath, fileName), (err) => {
        if (err) return reject(err);
        resolve();
      });
    })
  }

  async write(error, info = {}, options = {}) {
    const formattedInfo = this.format(error, info);
    const { NODE_ENV = 'development' } = process.env;
    if(NODE_ENV === 'development' && !this.hideConsoleLogger) console.error(formattedInfo);
    const { useWriteFile = false, ...fsOptions } = options;
    const fsFunctionToCall = useWriteFile ? fs.writeFile : fs.appendFile;
    return new Promise((resolve, reject) => {
      const path = this.getFullPath();
      const data = formattedInfo;
      const options = fsOptions;
      const callback = (err) => {
        if(err) return reject(err);
        resolve({
          fileName: this.getFormattedFileName(),
        });
      };
      fsFunctionToCall.apply(this, [path, data, options, callback]);
    });
  }

  /**
   * Setters and Getters
   */

  setFileName(fileName) {
    this.fileName = fileName;
  }

  getFileName() {
    return this.fileName;
  }

  getFormattedFileName() {
    let formattedFileName = this.fileName || 'log';
    if(this.useTimeStamp) formattedFileName = `${formattedFileName}_${DateHandler.toString()}`;
    return `${formattedFileName}.${this.fileExtension}`;
  }

  setFilePath(path) {
    this.filePath = path;
  }

  getFilePath() {
    return this.filePath;
  }

  getFullPath() {
    return path.join(this.filePath, this.getFormattedFileName());
  }

  isOldLog(logDate) {
    if(!this.keepLogDays || Number.isNaN(Number(this.keepLogDays))) return false;
    const DTFORMAT = 'YYYY-MM-DD';
    const NOW = moment(moment().format(DTFORMAT), DTFORMAT);
    const LOG_DATE = moment(logDate, DTFORMAT);
    return NOW.diff(LOG_DATE, 'days') >= Number(this.keepLogDays);
  }

  isLogFile(fileName) {
    return fileName && fileName.startsWith(this.fileName) && fileName.endsWith(this.fileExtension);
  }

  async clearLogs() {
    const DATE_PATTERN = /\d{4}-\d{2}-\d{2}/i;
    return new Promise((resolve, reject) => {
      if(!this.logFolder) {
        return console.error('<logFolder> not informed');
      }
      if (!fs.existsSync(this.logFolder)) {
        fs.mkdirSync(this.logFolder, { recursive: true });
      }
      fs.readdir(this.logFolder, async (err, files) => {
        if (err) return reject(err);
        const allPromiseRemoveFiles = [];
        files.forEach(fileName => {
          const [ logDate ] = fileName.match(DATE_PATTERN) || [];
          if(logDate && this.isOldLog(logDate) && this.isLogFile(fileName)) {
            allPromiseRemoveFiles.push(
              this.removeFile(this.logFolder, fileName)
            );
          }
        });
        const response = await Promise.allSettled(allPromiseRemoveFiles);
        const errors = response.filter(result => result.status  === 'rejected');
        if(errors.length) {
          const errorMessages = errors.map(err => err.reason);
          const message = `Ocorreram erros ao remover arquivos de log {${this.logFolder}}: ${errorMessages.join('\n')}`;
          console.error(message);
          return resolve({error: true, message})
        }
        resolve({error: false});
      });
    });
  }

  startCleanupTask(errorCallback = () => {}) {
    const promiseClearLog = this.clearLogs().catch(errorCallback);
    const period = this.keepLogDays && !isNaN(this.keepLogDays) ? ms(this.keepLogDays + 'd') : null;
    if(period) {
      this.cleanUpTaskIntervalId = this.cleanUpTaskIntervalId || setInterval(() => {
        this.clearLogs().catch(errorCallback);
      }, period);
    }
    return promiseClearLog;
  }
}

module.exports = Logger;