const ms = require('ms');
const MonitoringController = require('../controller/MonitoringController');
const Gzip = require('../functions/Gzip');
const SystemLogger = require('../logs/SystemLogger');
const AssistantConfig = require('../model/AssistantConfig');
const SocketRooms = require('./SocketRooms');

class MonitoringApacheCloud {
  constructor() {
    this.monitoringIntervals = {};
    this.monitoringParams = {};
    this.monitoringResult = {};
    this.io = null;
    this.defaultRoom = SocketRooms.master.monitoringCloudPage;
    this.timeLastConnectedUser = null;
  }
  hasActiveMonitoring() {
    return Object.keys(this.monitoringIntervals).length > 0;
  }
  getMonitoringResults() {
    return this.monitoringResult;
  }
  async getConsummers() {
    const consummers = await this.getRoom(this.defaultRoom).fetchSockets();
    return consummers;
  }
  async hasConsummers() {
    const consummers = await this.getConsummers();
    return consummers.length > 0;
  }
  setSocketIo(io) {
    this.io = io;
  }
  getSocketIo() {
    return this.io;
  }
  getRoom(roomName) {
    return this.getSocketIo().to(roomName);
  }
  async loadMonitoringParams() {
    const { rows } = await AssistantConfig.getMonitoringParamsByType({ TIPO: 'C' });
    if(!rows.length) return;
    this.monitoringParams = rows.reduce(( allParams, row ) => {
      row.COMANDO = Gzip.decode(row.COMANDO);
      allParams[row.ID] = row;
      return allParams;
    }, {});
  }
  updateMonitoringParams(config) {
    const { ID, NOME, DESCRICAO, ATIVO } = config;
    const room = this.getRoom(this.defaultRoom);
    if(ATIVO === 'S') {
      room.emit('startMonitoring', {ID: parseInt(ID, 10), NOME, DESCRICAO, ATIVO});
      this.startMonitoring(config);
    } else {
      room.emit('stopMonitoring', [parseInt(ID, 10)]);
      this.stopMonitoring(parseInt(ID, 10));
    }
  }
  stopMonitoring(ID) {
    const idsToStop = ID ? [ID] : Object.keys(this.monitoringIntervals);
    idsToStop.forEach(id => {
      clearInterval(this.monitoringIntervals[id]);
    });
  }
  async handleHttpRequest(config) {
    const { ID, NOME , COMANDO } = config;
    const response = await MonitoringController.execHttpRequest(ID, COMANDO.split(';'));
    const failResults = [];
    response.forEach(result => {
      const { ERROR, STATUS } = result;
      if (STATUS === 'N'){
        failResults.push(ERROR);
      }
    });
    if(failResults.length) {
      SystemLogger.write(`Apache Cloud - ERR: ${ID} - ${NOME} | Message: ${failResults.join('\n')}`);
    }
    this.monitoringResult[ID] = response;
    const socketResponse = { ID };
    socketResponse.STATUS = failResults.length ? 'N' : 'S';
    this.getRoom(this.defaultRoom).emit('apacheCloudResult', socketResponse);
  }
  registerEvents(socket, agent) {
    if(!['updater', 'monitor'].includes(agent)) {
      socket.on('detailMonitoringApache', ({ ID }) => {
        socket.emit('detailMonitoringApacheResult', this.monitoringResult[ID]);
      });
      socket.on('getLatestMonitoringResult', () => {});
    }
  }
  createIntervalMonitoring(config) {
    const { ID, INTERVALO } = config;
    if(ID in this.monitoringIntervals) {
      this.stopMonitoring(ID);
    }
    this.monitoringIntervals[ID] = setInterval(() => {
      this.handleHttpRequest(config);
    }, ms(`${parseInt(INTERVALO, 10)}s`));
  }
  async startMonitoring(config = null) {
    const hasActiveMonitoring = this.hasActiveMonitoring();
    const needCreateOrUpdateMonitoring = config || !hasActiveMonitoring;

    if(!needCreateOrUpdateMonitoring) return;
    const params = config ? [config] : Object.values(this.monitoringParams);
    params.forEach(config => {
      this.handleHttpRequest(config);
      this.createIntervalMonitoring(config);
    });
  }
}

module.exports = new MonitoringApacheCloud();